// beispiele/qteditor/MyWindow.cpp
#include "MyWindow.h"
#include <QApplication>
#include <QMessageBox>
#include <QTime>
#include <QTimer>
#include <QVariant>
#include <QFontComboBox>
#include <QCloseEvent>

// Neue Widget-Klasse vom eigentlichen Widget ableiten
MyWindow::MyWindow( QMainWindow *parent,
                    Qt::WindowFlags flags) :
                    QMainWindow(parent, flags)  {
    editor = new QTextEdit;

    // Das komplette Men zum Hauptprogramm
    QMenu *fileMenu = new QMenu(tr("&Datei"), this);
    menuBar()->addMenu(fileMenu);
    act1 = fileMenu->addAction(
       QIcon(":/images/page_white.png"), tr("&Neu"),
       this, SLOT(newFile()),
       QKeySequence(tr("Ctrl+N", "Datei|Neu")) );
    act1->setStatusTip(
       tr("Lscht den aktuellen Inhalt der Datei"));
    act2 = fileMenu->addAction(
       QIcon(":/images/folder_page_white.png"),
       tr("&ffnen..."), this, SLOT(openFile() ),
       QKeySequence(tr("Ctrl+O", "Datei|ffnen")));
    act2->setStatusTip(
       tr("ffnet eine Datei in den Texteditor"));
    fileMenu->addSeparator();
    act3 = fileMenu->addAction(
       QIcon(":/images/cancel.png"), tr("Be&enden"),
       qApp, SLOT(quit()),
       QKeySequence(tr("Ctrl+E", "Datei|Beenden")) );
    act3->setStatusTip(tr("Programm beenden"));
    QMenu *workMenu = new QMenu(
       tr("&Bearbeiten"), this);
    menuBar()->addMenu(workMenu);
    act4 = workMenu->addAction(
       tr("&Suchen"), this, SLOT(search()),
       QKeySequence(tr("Ctrl+S", "Bearbeiten|Suchen")));
    act4->setStatusTip(
       tr("Nach einer Stringfolge suchen") );
    // Men mit Linie von der Leiste Abnehmbar
    fileMenu->setSeparatorsCollapsible(true);

    // Mehrere Aktionen erzeugen
    QAction* underLineAct = new QAction(
       tr("&Unterstreichen"), this );
    underLineAct->setCheckable(true);
    underLineAct->setShortcut(tr("Ctrl+U"));
    underLineAct->setStatusTip(
       tr("Text unterstreichen") );
    underLineAct->setIcon(
       QIcon(":/images/text_underline.png"));
    connect( underLineAct, SIGNAL(triggered(bool)),
             editor, SLOT(setFontUnderline(bool)) );

    QAction *leftAlignAct = new QAction(
       tr("&Left Align"), this);
    leftAlignAct->setCheckable(true);
    leftAlignAct->setShortcut(tr("Ctrl+L"));
    leftAlignAct->setStatusTip(
       tr("Text links ausrichten"));
    leftAlignAct->setIcon(
       QIcon(":/images/text_align_left.png") );
    connect( leftAlignAct, SIGNAL(triggered()),
             this, SLOT(leftAlignment() ) );

    QAction *rightAlignAct = new QAction(
       tr("&Right Align"), this);
    rightAlignAct->setCheckable(true);
    rightAlignAct->setShortcut(tr("Ctrl+R"));
    rightAlignAct->setStatusTip(
       tr("Text rechts ausrichten") );
    rightAlignAct->setIcon(
       QIcon(":/images/text_align_right.png"));
    connect( rightAlignAct, SIGNAL(triggered()),
             this, SLOT(rightAlignment() ) );

    QAction *justifyAct = new QAction(
       tr("&Justify"), this);
    justifyAct->setCheckable(true);
    justifyAct->setShortcut(tr("Ctrl+J"));
    justifyAct->setStatusTip(
       tr("Text bndig ausrichten"));
    justifyAct->setIcon(
       QIcon(":/images/text_align_justify.png") );
    connect( justifyAct, SIGNAL(triggered()),
             this, SLOT(justifyAlignment() ) );

    QAction *centerAct = new QAction(
       tr("&Center"), this);
    centerAct->setCheckable(true);
    centerAct->setShortcut(tr("Ctrl+E"));
    centerAct->setStatusTip(
       tr("Text zentriert ausrichten") );
    centerAct->setIcon(
       QIcon(":/images/text_align_center.png") );
    connect( centerAct, SIGNAL(triggered()),
             this, SLOT(centerAlignment() ) );

    // Einige Aktionen zu einer Gruppe zusammenfassen
    QActionGroup* alignmentGroup = new QActionGroup(
       this );
    alignmentGroup->addAction(leftAlignAct);
    alignmentGroup->addAction(rightAlignAct);
    alignmentGroup->addAction(justifyAct);
    alignmentGroup->addAction(centerAct);
    leftAlignAct->setChecked(true);

    // Nochmals zwei weitere Aktionen
    QAction *InZoom = new QAction(
       tr("&Zoom in ..."), this );
    InZoom->setStatusTip(tr("Zoom in the text"));
    connect( InZoom, SIGNAL(triggered()),
             editor, SLOT(zoomIn() ) );
    QAction *OutZoom = new QAction(
       tr("&Zoom out ..."), this );
    OutZoom->setStatusTip(tr("Zoom out the text"));
    connect( OutZoom, SIGNAL(triggered() ),
             editor, SLOT(zoomOut() ) );

    // Ein Sub-Men "Format" bei "Bearbeiten" einfgen
    QMenu *formatMenu = workMenu->addMenu(
       tr("&Format") );
    formatMenu->addAction(underLineAct);
    formatMenu->addSeparator();
    formatMenu->addAction(leftAlignAct);
    formatMenu->addAction(rightAlignAct);
    formatMenu->addAction(justifyAct);
    formatMenu->addAction(centerAct);
    formatMenu->addSeparator();
    formatMenu->addAction(InZoom);
    formatMenu->addAction(OutZoom);
    workMenu->setTearOffEnabled(true);
    workMenu->setWindowTitle("Bearbeiten");

    // Zur Demonstration eine Aktion mit QVariant
    Start = new QAction(tr("&Programmstart ..."), this);
    Start->setStatusTip(tr("Zeit des Programmstarts"));
    Start->setData(QVariant(QTime::currentTime()));
    QFont font; font.setBold(true);
    Start->setFont(font);
    Start->setShortcut(tr("Ctrl+T"));
    Start->setShortcutContext(Qt::ApplicationShortcut);
    connect( Start, SIGNAL( triggered() ),
             this, SLOT( printStart() ) );
    workMenu->addAction(Start);

    // Eine neue Statusleiste erzeugen und zum
    // Hauptfenster hinzufgen
    (void*) statusBar ();
    // Permanente Meldung in der Statusleiste
    // Zeichen, Wrter, Zeilen
    sLabel = new QLabel;
    sLabel->setFrameStyle(
       QFrame::Panel | QFrame::Sunken );
    sLabel->setLineWidth(2);
    statusBar()->addPermanentWidget(sLabel);
    connect( editor, SIGNAL( textChanged() ),
             this, SLOT( updateStatusBar() ) );
    updateStatusBar();
    // Laufend aktuelle Uhrzeit in der Statusleiste
    time = new QLCDNumber;
    time->setFrameStyle(QFrame::Panel | QFrame::Sunken);
    time->setLineWidth(2);
    time->setSegmentStyle(QLCDNumber::Flat);
    statusBar()->addPermanentWidget(time);
    QTimer *timer = new QTimer(this);
    connect( timer, SIGNAL(timeout()),
             this, SLOT(updateTime()) );
    // Jede Sekunde updateTime() aufrufen
    timer->start(1000);
    updateTime();

    // Werkzeugleiste erzeugen
    toolFile = addToolBar(tr("Datei"));
    // Icons mit Text darunter anzeigen
    toolFile->setToolButtonStyle (
       Qt::ToolButtonTextUnderIcon );
    // Werkzeugleiste nur nach links
    // oder oben Verschiebbar
    toolFile->setAllowedAreas(
       Qt::LeftToolBarArea | Qt::TopToolBarArea );
    // Aktionen zur Werkzeugleiste hinzufgen
    toolFile->addAction(act1);
    toolFile->addAction(act2);
    toolFile->addAction(act3);
    // Noch eine Werkzeugleiste
    toolWork = addToolBar(tr("Bearbeiten"));
    // Werkzeugleiste unten anzeigen
    addToolBar(Qt::BottomToolBarArea, toolWork );
    // Position nicht mehr vernderbar
    toolWork->setMovable(false);
    // Aktionen zur Werkzeugleiste hinzufgen
    toolWork->addAction(underLineAct);
    toolWork->addSeparator ();
    toolWork->addAction(leftAlignAct);
    toolWork->addAction(rightAlignAct);
    toolWork->addAction(justifyAct);
    toolWork->addAction(centerAct);
    toolWork->addSeparator ();
    // Eine Schrift-Combobox zur Werkzeugleiste
    QFontComboBox *cFont = new QFontComboBox;
    toolWork->addWidget(cFont);
    connect(
      cFont, SIGNAL( currentFontChanged(const QFont& )),
      editor, SLOT(setCurrentFont ( const QFont&) ) );

    // Ein neues Dock-Widget-erzeugen
    QDockWidget *dock = new QDockWidget(
       tr("WYSIWYG-Editor"), this );
    // nur in den rechten und linken Bereich erlauben
    dock->setAllowedAreas(
       Qt::LeftDockWidgetArea|Qt::RightDockWidgetArea);
    // Widget fr das Dock-Widget erzeugen
    dock_editor = new QTextEdit;
    // Editor als Widget im Dock-Fenster verwenden
    dock->setWidget(dock_editor);
    // auf der rechten Seite zum Hauptfenster hinzufgen
    addDockWidget(Qt::RightDockWidgetArea, dock);
    connect( editor, SIGNAL(textChanged() ),
             this, SLOT( showHTML() ) );

    // Einstellungen des Programms speichern
    // Im Beispiel wird nur (ggf.) die zuletzt
    // geffnete Datei und die Gre des Fensters
    settings = new QSettings(
       tr("Pronix Inc."), tr("Qt-Texteditor") );
    settings->beginGroup("HauptFenster");
    QString fileName =
       settings->value("lastFile").toString();
    QSize size =
       settings->value("size", sizeHint()).toSize();
    if (!fileName.isEmpty()) {
       QFile file(fileName);
       if (file.open(QFile::ReadOnly | QFile::Text)) {
          editor->setPlainText(file.readAll());
          statusBar()->showMessage(
             tr("Zuletzt geffnete Datei "
                "erfolgreich geladen"));
          // Wieder setzen
          settings->setValue("lastFile", fileName );
       }
       else // falls die Datei nicht mehr existiert
          settings->setValue("lastFile", "");
    }
    else // Keine Datei gesetzt
       settings->setValue("lastFile", "");
    settings->endGroup();

    if( size.isNull() )
       resize(640, 480);
    else
       resize(size);
    setCentralWidget(editor);
    setWindowTitle("QMainWindow - Demo");
}

// Datei ffnen und im Editor anzeigen
void MyWindow::openFile() {
    QString fileName;
    fileName = QFileDialog::getOpenFileName(
       this, tr("Datei ffnen"), "",
       "C++ Dateien (*.cpp *.h);;"
       "Text-Dateien (*.txt);;"
       "Alle Dateien (*.*)" );
    if (!fileName.isEmpty()) {
       QFile file(fileName);
       if (file.open(QFile::ReadOnly | QFile::Text)) {
          editor->setPlainText(file.readAll());
          statusBar()->showMessage(
             tr("Datei erfolgreich geladen") );
          // Zuletzt geffnete Datei setzen
          settings->beginGroup("HauptFenster");
          settings->setValue("lastFile", fileName );
          settings->endGroup();
       }
    }
}

// Inhalt des Editors lschen
void MyWindow::newFile( ) {
   editor->clear();
}

// Im Editor nach einer bestimmten Textfolge suchen
void MyWindow::search( ) {
  bool ok;
  QString text = QInputDialog::getText(
     this, "Suchdialog", "Text zur Suche eingeben :",
     QLineEdit::Normal, "Suche eingeben", &ok );
  // Einen einfachen Fortschrittsbalken zur Suche
  // in der Statusleiste simulieren
  QProgressBar* pbar = new QProgressBar;
  // min. und max. Werte festlegen
  pbar->setRange( 0, 500 );
  statusBar()->addWidget(pbar);
  for (int i = 0; i < 500; i++) {
     pbar->setValue(i);
     for( int j=0; j < 12345678; ++j);
        //... copy one file
     }
  pbar->setValue(500);
  statusBar()->removeWidget(pbar);
  if (ok && !text.isEmpty())
     editor->find(text);
  statusBar()->showMessage(tr("Suche beendet"), 3000);
}

// Absatz links anordnen
void MyWindow::leftAlignment() {
  editor->setAlignment(Qt::AlignLeft);
}

// Absatz rechts anordnen
void MyWindow::rightAlignment() {
   editor->setAlignment(Qt::AlignRight);
}

// Absatz im Block ausrichten
void MyWindow::justifyAlignment() {
   editor->setAlignment(Qt::AlignJustify);
}

// Absatz zentrieren
void MyWindow::centerAlignment() {
   editor->setAlignment(Qt::AlignCenter);
}

// Kontextmen verwenden contextMenuEvent
// ist eine virtuelle Methode
void MyWindow::contextMenuEvent(
    QContextMenuEvent *event) {
   QMenu *menu = new QMenu(this);
   menu->addAction(act1);
   menu->addAction(act2);
   menu->addAction(act3);
   menu->exec(event->globalPos());
}

// Gibt die Uhrzeit des Programmstarts im Texteditor aus
void MyWindow::printStart() {
   QVariant data = Start->data();
   QTime time = data.toTime();
   // String daraus machen
   QString msg(
      "(Demonstriert setData()) Programmstart: ");
   msg.append(time.toString("hh:mm:ss"));
   editor->setText(msg);
}

// Zeichen, Wrter und Zeilen in der Statusleiste
void MyWindow::updateStatusBar() {
   QString str = editor->toPlainText();
   int count_char = str.length();
   int count_words = str.count(" ");
   int count_lines = str.count("\n");
   QString label =
    tr("  Zeichen: %1  Wrter: %2  Zeilen: %3 ")
   .arg(count_char).arg(count_words).arg(count_lines+1);
   sLabel->setText(label);
}

// Zeit in der Statusleiste
void MyWindow::updateTime() {
   // Aktuelles Datum und Uhrzeit
   QDateTime Time = QDateTime::currentDateTime();
   // Formatieren
   QString text = Time.toString("dd.MM.yyyy hh:mm:ss");
   // Anzahl der Ziffern setzen
   time->setNumDigits(text.size()+1);
   // Und alles Anzeigen
   time->display(text);
}

// HTML-Format im Dock-Widget anzeigen
void MyWindow::showHTML() {
    QString str = editor->toPlainText();
    dock_editor->setHtml(str);
}

// Wird ausgefhrt, bevor das Fenster geschlossen wird
void MyWindow::closeEvent(QCloseEvent *event)  {
   // Zuletzt geffnete Datei setzen
   settings->beginGroup("HauptFenster");
   settings->setValue("size", size() );
   settings->endGroup();
   event->accept();
}

